# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from collections import defaultdict
from pathlib import Path
from typing import Dict, Iterable, Iterator

from troubadix.plugin import LinterResult, LinterWarning


class Results:
    def __init__(self, ignore_warnings: bool = False) -> None:
        self.plugin_results: Dict[str, Iterable[LinterResult]] = defaultdict(
            list
        )
        self.has_plugin_results = False
        self._ignore_warnings = ignore_warnings

    def add_plugin_results(
        self, plugin_name: str, results: Iterator[LinterResult]
    ) -> "Results":
        if self._ignore_warnings:
            results = [
                result
                for result in results
                if not isinstance(result, LinterWarning)
            ]
        else:
            results = list(results)

        self.has_plugin_results = self.has_plugin_results or bool(results)
        self.plugin_results[plugin_name] += results
        return self

    def __bool__(self):
        return self.has_plugin_results


class FileResults(Results):
    """Class to store results from different plugins for a file"""

    def __init__(self, file_path: Path, ignore_warnings: bool = False):
        self.file_path = file_path
        super().__init__(ignore_warnings)


def resultsdict():
    return defaultdict(int)


class ResultCounts:
    """Class that counts different types of results of different plugins"""

    def __init__(self):
        self.result_counts = defaultdict(resultsdict)
        self.error_count = 0
        self.warning_count = 0
        self.fix_count = 0

    def add_error(self, plugin: str):
        self.error_count += 1
        self.result_counts[plugin]["error"] += 1

    def add_warning(self, plugin: str):
        self.warning_count += 1
        self.result_counts[plugin]["warning"] += 1

    def add_fix(self, plugin: str):
        self.fix_count += 1
        self.result_counts[plugin]["fix"] += 1



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
from argparse import ArgumentParser, Namespace
from collections import defaultdict
from datetime import datetime, timedelta
from pathlib import Path
from typing import Iterable, List, Optional, Tuple

from pontos.terminal.terminal import ConsoleTerminal

from troubadix.helper import CURRENT_ENCODING
from troubadix.helper.patterns import (
    ScriptTag,
    SpecialScriptTag,
    get_script_tag_pattern,
    get_special_script_tag_pattern,
)

SOLUTION_TYPE_NONE_AVAILABLE = "NoneAvailable"
CVSS_DETECTION_SCRIPT = "0.0"

SOLUTION_PATTERN = get_script_tag_pattern(ScriptTag.SOLUTION)
SOLUTION_DATE_PATTERN = re.compile(r"as\s+of\s*(?P<date>.+?)\.\s*", re.DOTALL)
SOLUTION_TYPE_PATTERN = get_script_tag_pattern(ScriptTag.SOLUTION_TYPE)
CREATION_DATE_PATTERN = get_script_tag_pattern(ScriptTag.CREATION_DATE)
CVSS_PATTERN = get_script_tag_pattern(ScriptTag.CVSS_BASE)
OID_PATTERN = get_special_script_tag_pattern(SpecialScriptTag.OID)

SOLUTION_DATE_FORMATS = ["%d %B, %Y", "%d %b, %Y", "%Y/%m/%d"]
CREATION_DATE_FORMAT = "%Y-%m-%d"

MONTH_AS_DAYS = 365 / 12


def directory_type(string: str) -> Path:
    file_path = Path(string)
    if not file_path.is_dir():
        raise ValueError(f"{string} is not a directory.")
    return file_path


def parse_solution_date(date_string: str) -> datetime:
    """Convert date string to date trying different formats"""

    date_string = re.sub(
        r"(?P<date>.\d{1,2})(st|nd|rd|th)", r"\g<date>", date_string
    )

    for strptime in SOLUTION_DATE_FORMATS:
        try:
            date = datetime.strptime(date_string, strptime)

            return date
        except ValueError:
            pass

    return None


def parse_args() -> Namespace:
    parser = ArgumentParser(
        description="Check VTs for solution type 'NoneAvailable'",
    )

    parser.add_argument(
        "-d",
        "--directory",
        dest="directory",
        type=directory_type,
        help="Specify the directory to scan for nasl scripts",
    )

    parser.add_argument(
        "-m",
        "--milestones",
        dest="milestones",
        help="Defines the milestones for which to report VTs "
        "without a solution for, in months. VTs with no solution newer "
        "than the smallest milestone will not be reported.",
        nargs="+",
        type=int,
        default=[12, 6, 1],
    )

    parser.add_argument(
        "-t",
        "--threshold",
        dest="threshold",
        type=int,
        default=12,
        help="The threshold after which to assume no solution "
        "will be provided anymore",
    )

    parser.add_argument(
        "-s",
        "--snooze",
        dest="snooze",
        type=int,
        default=1,
        help="The duration, in months, to suppress reporting VTs after, based "
        "on the date stated in the solution text.",
    )

    return parser.parse_args()


def check_skip_script(file_content: str) -> bool:
    solution_type = SOLUTION_TYPE_PATTERN.search(file_content)
    if (
        solution_type
        and solution_type.group("value") != SOLUTION_TYPE_NONE_AVAILABLE
    ):
        return True

    cvss = CVSS_PATTERN.search(file_content)
    if cvss and cvss.group("value") == CVSS_DETECTION_SCRIPT:
        return True

    return False


def extract_tags(content: str) -> Optional[Tuple[datetime, datetime, str]]:
    solution_match = SOLUTION_PATTERN.search(content)
    if not solution_match:
        return None

    date_match = SOLUTION_DATE_PATTERN.search(solution_match.group("value"))
    if not date_match:
        return None

    solution_date = parse_solution_date(date_match.group("date"))
    if not solution_date:
        return None

    creation_match = CREATION_DATE_PATTERN.search(content)
    if not creation_match:
        return None

    creation_date = datetime.strptime(
        creation_match.group("value")[:10], CREATION_DATE_FORMAT
    )

    oid_match = OID_PATTERN.search(content)
    if not oid_match:
        return None

    oid = oid_match.group("value")

    return solution_date, creation_date, oid


def check_no_solutions(
    files: Iterable[Path], milestones: List[int], snooze_duration: int
) -> List[Tuple[int, List[Tuple[Path, str, datetime, datetime]]]]:
    summary = defaultdict(list)

    for nasl_file in files:
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        if check_skip_script(content):
            continue

        extracted_tags = extract_tags(content)
        if not extracted_tags:
            raise ValueError(f"Could not extract tags from {nasl_file}")

        solution_date, creation_date, oid = extracted_tags

        solution_diff = datetime.now() - solution_date

        if solution_diff < timedelta(days=snooze_duration * MONTH_AS_DAYS):
            continue

        creation_diff = datetime.now() - creation_date

        for milestone in milestones:
            milestone_delta = timedelta(days=milestone * MONTH_AS_DAYS)
            if creation_diff >= milestone_delta:
                summary[milestone].append(
                    (nasl_file, oid, solution_date, creation_date)
                )
                break

    return sorted(
        [(milestone, vts) for milestone, vts, in summary.items()],
        key=lambda tuple: tuple[0],
        reverse=True,
    )


def print_info(
    term: ConsoleTerminal,
    milestones: List[int],
    threshold: int,
    snooze: int,
    root: Path,
):
    term.bold_info("Report VTs with solution type 'NoneAvailable'")
    with term.indent():
        term.print(f"Root directory: {root}")
        milestone_str = ", ".join(str(milestone) for milestone in milestones)
        term.print(f"Milestones: {milestone_str} months")
        term.print(f"Expect no solution threshold: {threshold} months")
        term.print(f"Snooze duration: {snooze} months")


def print_report(
    term: ConsoleTerminal,
    summary: Iterable[Tuple[int, List[Tuple[Path, str, datetime, datetime]]]],
    threshold: int,
    root: Path,
    total: int,
):
    term.info(f"Total VTs without solution: {total}\n")

    for milestone, vts in summary:

        vts = sorted(vts)

        if milestone >= threshold:
            term.bold_info(
                f"{len(vts)} VTs with no solution "
                f"for more than {milestone} month(s).\n"
                "No solution should be expected at this point. "
            )
        else:
            term.bold_info(
                f"{len(vts)} VTs with no solution for "
                f"more than {milestone} month(s)"
            )

        for vt, oid, solution, creation in vts:
            term.info(str(vt.relative_to(root)))

            with term.indent():
                term.print(f"OID: {oid}")
                term.print(f"Created: {creation.strftime('%Y-%m-%d')}")
                term.print(
                    f"Last solution update: {solution.strftime('%Y-%m-%d')}"
                )

        term.print()


def main():

    try:

        arguments = parse_args()

        root = arguments.directory or Path.cwd()

        files = root.rglob("*.nasl")

        milestones = sorted(arguments.milestones, reverse=True)

        term = ConsoleTerminal()

        print_info(
            term, milestones, arguments.threshold, arguments.snooze, root
        )

        summary = check_no_solutions(files, milestones, arguments.snooze)

        found_vts = sum(len(entries) for _, entries in summary)

        print_report(term, summary, arguments.threshold, root, found_vts)

        sys.exit(1 if found_vts > 0 else 0)

    # pylint: disable=broad-except
    except Exception as e:

        print(f"troubadix-no-solution encountered an error: {e}")

        sys.exit(2)


if __name__ == "__main__":
    main()



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=fixme

import re
from typing import Iterator

from troubadix.helper.patterns import _get_special_script_tag_pattern
from troubadix.plugin import FilePlugin, LinterError, LinterResult

DEPENDENCY_ENTRY_PATTERN = re.compile(
    r'(?P<quote>[\'"])(?P<value>[^\'"]*)(?P=quote)'
)
WHITESPACE_PATTERN = re.compile(r"\s")


class CheckMalformedDependencies(FilePlugin):
    name = "check_malformed_dependencies"

    def run(
        self,
    ) -> Iterator[LinterResult]:
        """This script checks whether the files used in script_dependencies()
        exist on the local filesystem.
        An error will be thrown if a dependency could not be found.
        """

        if self.context.nasl_file.suffix == ".inc":
            return

        dependencies_pattern = _get_special_script_tag_pattern(
            "dependencies", flags=re.DOTALL | re.MULTILINE
        )

        file_content = self.context.file_content

        matches = dependencies_pattern.finditer(file_content)

        for match in matches:
            if not match:
                continue

            tag_value = (
                f'"{match.group("value")}"' if match.group("value") else ""
            )

            dependency_entries = DEPENDENCY_ENTRY_PATTERN.finditer(tag_value)

            for dependency_entry in dependency_entries:

                if not dependency_entry:
                    continue

                dependency_value = dependency_entry.group("value")

                if "," in dependency_value:
                    yield LinterError(
                        "The script dependency value is malformed and "
                        "contains a comma in the dependency value: "
                        f"'{dependency_value}'",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )

                if WHITESPACE_PATTERN.search(dependency_value):
                    yield LinterError(
                        "The script dependency value is malformed and "
                        "contains whitespace within the dependency value: "
                        f"'{dependency_value}'",
                        file=self.context.nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from troubadix.helper.patterns import (
    ScriptTag,
    SpecialScriptTag,
    get_script_tag_pattern,
    get_script_tag_patterns,
    get_special_script_tag_pattern,
    get_special_script_tag_patterns,
)


class ScriptTagPatternsTestCase(unittest.TestCase):
    def test_same_patterns_instance(self):
        patterns1 = get_script_tag_patterns()
        patterns2 = get_script_tag_patterns()

        self.assertIs(patterns1, patterns2)

    def test_same_pattern_instance(self):
        pattern1 = get_script_tag_pattern(ScriptTag.AFFECTED)
        pattern2 = get_script_tag_pattern(ScriptTag.AFFECTED)

        self.assertIs(pattern1, pattern2)


class SpecialScriptTagPatternsTestCase(unittest.TestCase):
    def test_same_patterns_instance(self):
        patterns1 = get_special_script_tag_patterns()
        patterns2 = get_special_script_tag_patterns()

        self.assertIs(patterns1, patterns2)

    def test_same_pattern_instance(self):
        pattern1 = get_special_script_tag_pattern(
            SpecialScriptTag.ADD_PREFERENCE
        )
        pattern2 = get_special_script_tag_pattern(
            SpecialScriptTag.ADD_PREFERENCE
        )

        self.assertIs(pattern1, pattern2)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Iterable, List

from pontos.terminal import Terminal

from troubadix.helper.helper import get_path_from_root
from troubadix.plugin import LinterError, LinterFix, LinterResult, LinterWarning
from troubadix.plugins import Plugins
from troubadix.results import FileResults, ResultCounts, Results


class Reporter:
    def __init__(
        self,
        term: Terminal,
        root: Path,
        *,
        fix: bool = False,
        log_file: Path = None,
        log_file_statistic: Path = None,
        statistic: bool = True,
        verbose: int = 0,
        ignore_warnings: bool = False,
    ) -> None:
        self._term = term
        self._log_file = log_file
        self._log_file_statistic = log_file_statistic
        self._statistic = statistic
        self._verbose = verbose
        self._fix = fix
        self._files_count = 0
        self._root = root
        self._ignore_warnings = ignore_warnings
        self._result_counts = ResultCounts()

    def set_files_count(self, count: int):
        self._files_count = count

    def get_error_count(self) -> int:
        return self._result_counts.error_count

    def _report_warning(self, message: str) -> None:
        self._term.warning(message)
        self._log_append(f"\t\t{message}".replace("\n", "\n\t\t"))

    def _report_error(self, message: str) -> None:
        self._term.error(message)
        self._log_append(f"\t\t{message}".replace("\n", "\n\t\t"))

    def report_info(self, message: str) -> None:
        """Report an info message"""
        self._term.info(message)
        self._log_append(f"\t{message}")

    def _report_bold_info(self, message: str) -> None:
        self._term.bold_info(message)
        self._log_append(f"\n\n{message}")

    def _report_ok(self, message: str) -> None:
        self._term.ok(message)
        self._log_append(f"\t\t{message}".replace("\n", "\n\t\t"))

    def _process_plugin_results(
        self, plugin_name: str, plugin_results: List[LinterResult]
    ):
        """Process the results of a plugin: Print/Log results if
        verbosity/logging fits and count the results"""
        if plugin_results and self._verbose > 0:
            self.report_info(f"Results for plugin {plugin_name}")
        elif self._verbose > 2:
            self._report_ok(f"No results for plugin {plugin_name}")

        # add the results to the statistic and print/log them
        with self._term.indent():
            for plugin_result in plugin_results:
                if isinstance(plugin_result, LinterError):
                    self._result_counts.add_error(plugin_name)
                    report = self._report_error
                elif isinstance(plugin_result, LinterWarning):
                    self._result_counts.add_warning(plugin_name)
                    report = self._report_warning
                elif isinstance(plugin_result, LinterFix):
                    self._result_counts.add_fix(plugin_name)
                    report = self._report_ok
                elif isinstance(plugin_result, LinterResult):
                    report = self._report_ok

                if self._verbose > 0:
                    report(plugin_result.message)

    def report_by_plugin(self, results: Results) -> None:
        """Print/log results per plugins

        Arguments:
            results    a results object
        """
        for (
            plugin_name,
            plugin_results,
        ) in results.plugin_results.items():
            if results and self._verbose > 0 or self._verbose > 1:
                self._report_bold_info(f"Run plugin {plugin_name}")

            with self._term.indent():
                self._process_plugin_results(plugin_name, plugin_results)

    def report_by_file_plugin(
        self, file_results: FileResults, pos: int
    ) -> None:
        """Print/log the results of all plugins for a specific file

        Arguments:
            file_results    a file results object
            pos             the absolute file number in relation
                            to the whole file count
        """
        if file_results and self._verbose > 0 or self._verbose > 1:
            # only print the part "common/some_nasl.nasl"
            from_root_path = get_path_from_root(
                file_results.file_path, self._root
            )
            self._report_bold_info(
                f"Checking {from_root_path} ({pos}/{self._files_count})"
            )

        with self._term.indent():
            for (
                plugin_name,
                plugin_results,
            ) in file_results.plugin_results.items():
                self._process_plugin_results(plugin_name, plugin_results)

    def report_plugin_overview(
        self,
        plugins: Plugins,
        excluded: Iterable[str],
        included: Iterable[str],
    ) -> None:
        """Print/log an overview, which plugins are in-/excluded and which one
        will be executed"""
        if self._verbose > 2:
            if excluded:
                self.report_info(f"Excluded Plugins: {', '.join(excluded)}")

            if included:
                self.report_info(f"Included Plugins: {', '.join(included)}")

            self.report_info(
                f"Running plugins: {', '.join([p.name for p in plugins])}"
            )

    def report_statistic(self) -> None:
        """Print a Error/Warning summary from the different plugins"""
        if not self._statistic:
            return

        if self._fix and self._ignore_warnings:
            line = f"{'Plugin':48} {'  Errors':8} {'   Fixes':8}"
            length = "-" * 67
        elif self._fix:
            line = f"{'Plugin':48} {'  Errors':8} {'Warnings':8} {'   Fixes':8}"
            length = "-" * 75
        elif self._ignore_warnings:
            line = f"{'Plugin':48} {'  Errors':8}"
            length = "-" * 59
        else:
            line = f"{'Plugin':48} {'  Errors':8} {'Warnings':8}"
            length = "-" * 67

        self._term.print(line)
        self._log_statistic_append(line)
        self._term.print(length)
        self._log_statistic_append(length)

        for (plugin, count) in self._result_counts.result_counts.items():
            if self._fix and self._ignore_warnings:
                line = f"{plugin:48} {count['error']:8} {count['fix']:8}"
            elif self._fix:
                line = (
                    f"{plugin:48} {count['error']:8} {count['warning']:8}"
                    f" {count['fix']:8}"
                )
            elif self._ignore_warnings:
                line = f"{plugin:48} {count['error']:8}"
            else:
                line = f"{plugin:48} {count['error']:8} {count['warning']:8}"

            if count["error"] > 0:
                self._term.error(line)
                self._log_statistic_append(line)
            else:
                self._term.warning(line)
                self._log_statistic_append(line)

        self._term.print(length)
        self._log_statistic_append(length)

        if self._fix and self._ignore_warnings:
            line = (
                f"{'sum':48} {self._result_counts.error_count:8}"
                f" {self._result_counts.fix_count:8}"
            )
        elif self._fix:
            line = (
                f"{'sum':48} {self._result_counts.error_count:8}"
                f" {self._result_counts.warning_count:8}"
                f" {self._result_counts.fix_count:8}"
            )
        elif self._ignore_warnings:
            line = f"{'sum':48} {self._result_counts.error_count:8}"
        else:
            line = (
                f"{'sum':48} {self._result_counts.error_count:8}"
                f" {self._result_counts.warning_count:8}"
            )

        self._term.info(line)
        self._log_statistic_append(line)

    def _log_append(self, message: str):
        if self._log_file:
            with self._log_file.open(mode="a", encoding="utf-8") as f:
                f.write(f"{message}\n")

    def _log_statistic_append(self, message: str):
        if self._log_file_statistic:
            with self._log_file_statistic.open(mode="a", encoding="utf-8") as f:
                f.write(f"{message}\n")

    def plugin_not_found(self, plugin_name):
        self._report_error(f"Plugin {plugin_name} is not existing.")

    def plugin_unknown(self, plugin_name):
        self._report_error(f"Plugin {plugin_name} can not be read.")



# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterable, Iterator

from troubadix.helper import is_ignore_file
from troubadix.plugin import LineContentPlugin, LinterError, LinterResult

_IGNORE_FILES = (
    "sw_telnet_os_detection.nasl",
    "gb_hp_comware_platform_detect_snmp.nasl",
    "gb_hirschmann_telnet_detect.nasl",
)


class CheckCopyrightYear(LineContentPlugin):
    """This steps checks if a VT contains a Copyright statement containing a
    year not matching the year defined in the creation_date statement like
    script_tag(name:"creation_date", value:"2017-
    """

    name = "check_copyright_year"

    def check_lines(
        self,
        nasl_file: Path,
        lines: Iterable[str],
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        creation_date = ""
        creation_year = ""
        copyrights = []

        for i, line in enumerate(lines, 1):
            if "creation_date" in line:
                expre = re.search(r'value\s*:\s*"(.*)"', line)
                if expre is not None and expre.group(1) is not None:
                    creation_date = expre.group(1)
                    expre = re.search(r"^([0-9]+)-", creation_date)
                    if expre is not None and expre.group(1) is not None:
                        creation_year = expre.group(1)

            copyright_match = re.search(
                r"(# |script_copyright.*)[Cc]opyright \([Cc]\) ([0-9]+)",
                line,
            )
            if (
                copyright_match is not None
                and copyright_match.group(2) is not None
                and not is_ignore_file(nasl_file, _IGNORE_FILES)
            ):
                copyrights.append((i, line, copyright_match.group(2)))

        if not creation_year:
            yield LinterError(
                "Missing creation_date statement in VT",
                file=nasl_file,
                plugin=self.name,
            )

        # key is the line where the copyright is found, value the year found
        # within that line
        for nr, line, copyright_year in copyrights:
            if copyright_year != creation_year:
                if "pre2008" in str(nasl_file):
                    if copyright_year > creation_year:
                        yield LinterError(
                            "VT contains a Copyright year not matching "
                            f"the year {creation_year} at line {nr}",
                            file=nasl_file,
                            plugin=self.name,
                            line=nr,
                        )
                else:
                    yield LinterError(
                        "VT contains a Copyright year not matching "
                        f"the year {creation_year} at line {nr}",
                        file=nasl_file,
                        plugin=self.name,
                        line=nr,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckTabs(FilePlugin):
    name = "check_tabs"

    def run(self) -> Iterator[LinterResult]:
        """This script checks if a VT is using one or
        more tabs instead of spaces."""
        for nr, line in enumerate(self.context.lines, 1):
            if "\t" in line:
                yield LinterError(
                    "VT uses tabs instead of spaces.",
                    file=self.context.nasl_file,
                    plugin=self.name,
                    line=nr,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.helper.patterns import get_common_tag_patterns
from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckDoubleEndPoints(FilePlugin):
    name = "check_double_end_points"

    def run(
        self,
    ) -> Iterator[LinterResult]:
        """This script checks if a VT is using one or more doubled end point
        in a script_tag like e.g.:

            script_tag(name:"insight", value:"My insight..");

            or:

            script_tag(name:"insight", value:"My insight.
            .");
        """

        if self.context.nasl_file.suffix == ".inc":
            return

        tag_matches = get_common_tag_patterns().finditer(
            self.context.file_content
        )

        if tag_matches is not None:
            for tag_match in tag_matches:
                if tag_match:
                    doubled_end_points_match = re.search(
                        r'.*\.\s*\.(?P<quote>[\'"])\s*\)\s*;',
                        tag_match.group(0),
                        re.MULTILINE,
                    )
                    if doubled_end_points_match:

                        # Valid string used in a few VTs.
                        if (
                            'and much more...");'
                            in doubled_end_points_match.group(0)
                        ):
                            continue

                        yield LinterError(
                            f"The script tag '{tag_match.group('name')}' "
                            "is ending with two or more points: "
                            f"'{tag_match.group('value')}'.",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" updating the modification time in VTs that have been touched/edited """

import datetime
import re
import sys
from argparse import ArgumentParser, ArgumentTypeError, Namespace
from pathlib import Path
from typing import Iterable, Sequence

from pontos.terminal import Terminal
from pontos.terminal.terminal import ConsoleTerminal

from troubadix.helper import CURRENT_ENCODING
from troubadix.helper.patterns import (
    LAST_MODIFICATION_ANY_VALUE_PATTERN,
    SCRIPT_VERSION_ANY_VALUE_PATTERN,
)
from troubadix.troubadix import from_file


def existing_file_type(string: str) -> Path:
    file_path = Path(string)
    if not file_path.exists():
        raise ArgumentTypeError(f'File "{string}" does not exist.')
    if not file_path.is_file():
        raise ArgumentTypeError(f'"{string}" is not a file.')
    return file_path


def update(nasl_file: Path, terminal: Terminal):
    file_content = nasl_file.read_text(encoding=CURRENT_ENCODING)

    # update modification date
    tag_template = 'script_tag(name:"last_modification", value:"{date}");'

    match_last_modification_any_value = re.search(
        pattern=LAST_MODIFICATION_ANY_VALUE_PATTERN,
        string=file_content,
    )

    if not match_last_modification_any_value:
        terminal.warning(
            f'Ignoring "{nasl_file}" because it is missing a '
            "last_modification tag."
        )
        return

    now = datetime.datetime.now(datetime.timezone.utc)
    # get that date formatted correctly:
    # "2021-03-24 10:08:26 +0000 (Wed, 24 Mar 2021)"
    correctly_formatted_datetime = f"{now:%Y-%m-%d %H:%M:%S %z (%a, %d %b %Y)}"

    file_content = file_content.replace(
        match_last_modification_any_value.group(0),
        tag_template.format(date=correctly_formatted_datetime),
    )

    # update script version
    script_version_template = 'script_version("{date}");'

    match_script_version = re.search(
        pattern=SCRIPT_VERSION_ANY_VALUE_PATTERN,
        string=file_content,
    )
    if not match_script_version:
        terminal.warning(
            f'Ignoring "{nasl_file}" because it is missing a ' "script_version."
        )
        return

    # get that date formatted correctly:
    # "2021-03-24T10:08:26+0000"
    correctly_formatted_version = f"{now:%Y-%m-%dT%H:%M:%S%z}"

    new_file_content = file_content.replace(
        match_script_version.group(0),
        script_version_template.format(date=correctly_formatted_version),
    )

    nasl_file.write_text(new_file_content, encoding=CURRENT_ENCODING)


def parse_args(args: Sequence[str] = None) -> Namespace:
    parser = ArgumentParser(
        description="Update script_version and last_modification tags"
    )
    what_group = parser.add_mutually_exclusive_group(required=True)
    what_group.add_argument(
        "--files",
        nargs="+",
        type=existing_file_type,
        help="List of files that should be updated",
    )
    what_group.add_argument(
        "--from-file",
        type=existing_file_type,
        help=(
            "Pass a file that contains a List of files "
            "containing paths to files, that should be "
            "updated. Files should be separated by newline."
        ),
    )
    return parser.parse_args(args)


def main() -> int:
    parsed_args = parse_args()
    terminal = ConsoleTerminal()

    if parsed_args.from_file:
        files = from_file(include_file=parsed_args.from_file, term=terminal)
    elif parsed_args.files:
        files: Iterable[Path] = parsed_args.files
    else:
        # will not happen
        sys.exit(1)

    for nasl_file in files:
        if nasl_file.suffix != ".nasl":
            terminal.warning(f'Skipping "{nasl_file}". Not a nasl file.')
            continue

        terminal.info(f'Updating "{nasl_file}"')
        update(nasl_file, terminal)
    return 0


if __name__ == "__main__":
    sys.exit(main())



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import signal
from multiprocessing import Pool
from pathlib import Path
from typing import Iterable

from troubadix.helper.patterns import (
    init_script_tag_patterns,
    init_special_script_tag_patterns,
)
from troubadix.plugin import FilePluginContext, FilesPluginContext, Plugin
from troubadix.plugins import StandardPlugins
from troubadix.reporter import Reporter
from troubadix.results import FileResults, Results

CHUNKSIZE = 1  # default 1


class TroubadixException(Exception):
    """Generic Exception for Troubadix"""


def initializer():
    """Ignore CTRL+C in the worker process."""
    signal.signal(signal.SIGINT, signal.SIG_IGN)


class Runner:
    def __init__(
        self,
        n_jobs: int,
        reporter: Reporter,
        *,
        root: Path,
        excluded_plugins: Iterable[str] = None,
        included_plugins: Iterable[str] = None,
        fix: bool = False,
        ignore_warnings: bool = False,
    ) -> bool:
        # plugins initialization
        self.plugins = StandardPlugins(excluded_plugins, included_plugins)

        self._excluded_plugins = excluded_plugins
        self._included_plugins = included_plugins

        self._reporter = reporter
        self._n_jobs = n_jobs
        self._root = root
        self._fix = fix
        self._ignore_warnings = ignore_warnings

        init_script_tag_patterns()
        init_special_script_tag_patterns()

    def _check(self, plugin: Plugin, results: Results) -> Results:
        """Run a single plugin and collect the results"""
        results.add_plugin_results(plugin.name, plugin.run())

        if self._fix:
            results.add_plugin_results(plugin.name, plugin.fix())

        return results

    def _check_files(self, plugin: Plugin) -> Results:
        """Run a files plugin and collect the results"""
        results = Results(ignore_warnings=self._ignore_warnings)
        return self._check(plugin, results)

    def _check_file(self, file_path: Path) -> FileResults:
        """Run all file plugins on a single file and collect the results"""
        results = FileResults(file_path, ignore_warnings=self._ignore_warnings)
        context = FilePluginContext(
            root=self._root, nasl_file=file_path.resolve()
        )

        for plugin_class in self.plugins.file_plugins:
            plugin = plugin_class(context)

            self._check(plugin, results)

        return results

    def _run_pooled(self, files: Iterable[Path]):
        """Run all plugins that check single files"""
        self._reporter.set_files_count(len(files))
        with Pool(processes=self._n_jobs, initializer=initializer) as pool:
            try:
                # run files plugins
                context = FilesPluginContext(root=self._root, nasl_files=files)
                files_plugins = [
                    plugin_class(context)
                    for plugin_class in self.plugins.files_plugins
                ]

                for results in pool.imap_unordered(
                    self._check_files, files_plugins, chunksize=CHUNKSIZE
                ):
                    self._reporter.report_by_plugin(results)

                # run file plugins
                for i, results in enumerate(
                    iterable=pool.imap_unordered(
                        self._check_file, files, chunksize=CHUNKSIZE
                    ),
                    start=1,
                ):
                    self._reporter.report_by_file_plugin(
                        file_results=results, pos=i
                    )

            except KeyboardInterrupt:
                pool.terminate()
                pool.join()

    def run(self, files: Iterable[Path]) -> bool:
        """The function that should be executed to run
        the Plugins over all files"""
        if not len(self.plugins):
            raise TroubadixException("No Plugin found.")

        # print plugins that will be executed
        self._reporter.report_plugin_overview(
            plugins=self.plugins,
            excluded=self._excluded_plugins,
            included=self._included_plugins,
        )

        start = datetime.datetime.now()
        self._run_pooled(files)

        self._reporter.report_info(
            f"Time elapsed: {datetime.datetime.now() - start}"
        )
        self._reporter.report_statistic()

        # Return true if no error exists
        return self._reporter.get_error_count() == 0



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.script_family import CheckScriptFamily

from . import PluginTestCase


class CheckScriptFamilyTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base_vector", value:"AV:N/A:N");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_family("FreeBSD Local Security Checks");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptFamily(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckScriptFamily(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_script_family(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base_vector", value:"AV:N/A:N");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_family("TestTest");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptFamily(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Invalid or misspelled script family 'TestTest'", results[0].message
        )

    def test_script_family2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base_vector", value:"AV:N/A:N");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptFamily(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual("No script family exist", results[0].message)

    def test_script_family3(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base_vector", value:"AV:N/A:N");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_family("???\\");\n'
            'script_family("???\\");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptFamily(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "More then one script family exist", results[0].message
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import unittest

from troubadix.helper.linguistic_exception_handler import (
    CompositeCheck,
    FileCheck,
    FilePatternCheck,
    FilesCheck,
    PatternCheck,
    PatternInFileCheck,
    PatternInFilePatternCheck,
    PatternInFilesCheck,
    PatternsCheck,
    PatternsInFileCheck,
    PatternsInFilePatternCheck,
    TextCheck,
    TextInFileCheck,
    handle_linguistic_checks,
)


class LinguisticExceptionHandlerTestCase(unittest.TestCase):
    def test_file_check(self):
        check = FileCheck("test")

        self.assertEqual(check.execute("test", None), True)
        self.assertEqual(check.execute("hello", None), False)

    def test_files_check(self):
        check = FilesCheck(["test1", "test2"])

        self.assertEqual(check.execute("test1", None), True)
        self.assertEqual(check.execute("test2", None), True)
        self.assertEqual(check.execute("hello", None), False)

    def test_file_pattern_check(self):
        check = FilePatternCheck(r"test\d")

        self.assertEqual(check.execute("test1", None), True)
        self.assertEqual(check.execute("test2", None), True)
        self.assertEqual(check.execute("Hello1", None), False)

    def test_file_pattern_flag_check(self):
        check = FilePatternCheck(r"test\d", re.IGNORECASE)

        self.assertEqual(check.execute("TEST1", None), True)
        self.assertEqual(check.execute("TEST2", None), True)
        self.assertEqual(check.execute("HELLO1", None), False)

    def test_text_check(self):
        check = TextCheck("test")

        self.assertEqual(check.execute(None, "test"), True)
        self.assertEqual(check.execute(None, "hello"), False)

    def test_pattern_check(self):
        check = PatternCheck(r"test\d")

        self.assertEqual(check.execute(None, "test1"), True)
        self.assertEqual(check.execute(None, "test2"), True)
        self.assertEqual(check.execute(None, "Hello1"), False)

    def test_pattern_flag_check(self):
        check = PatternCheck(r"test\d", re.IGNORECASE)

        self.assertEqual(check.execute(None, "TEST1"), True)
        self.assertEqual(check.execute(None, "TEST2"), True)
        self.assertEqual(check.execute(None, "HELLO1"), False)

    def test_patterns_check(self):
        check = PatternsCheck([r"test\d", r"foo\d"])

        self.assertEqual(check.execute(None, "test1"), True)
        self.assertEqual(check.execute(None, "foo2"), True)
        self.assertEqual(check.execute(None, "HELLO1"), False)

    def test_patterns_tuple_check(self):
        check = PatternsCheck([(r"test\d", 0), (r"foo\d", re.IGNORECASE)])

        self.assertEqual(check.execute(None, "test1"), True)
        self.assertEqual(check.execute(None, "FOO2"), True)
        self.assertEqual(check.execute(None, "HELLO1"), False)

    def test_composite_check(self):
        check = CompositeCheck(FileCheck("test"), TextCheck("test"))

        self.assertEqual(check.execute("test1", "test2"), True)
        self.assertEqual(check.execute("hello", "test2"), False)
        self.assertEqual(check.execute("test1", "hello"), False)
        self.assertEqual(check.execute("hello", "hello"), False)

    def test_text_in_file_check(self):
        check = TextInFileCheck("test", "test")

        self.assertEqual(check.execute("test1", "test2"), True)
        self.assertEqual(check.execute("hello", "test2"), False)
        self.assertEqual(check.execute("test1", "hello"), False)
        self.assertEqual(check.execute("hello", "hello"), False)

    def test_pattern_in_file_check(self):
        check = PatternInFileCheck("test", r"(test|hello)\d")

        self.assertEqual(check.execute("test1", "test2"), True)
        self.assertEqual(check.execute("test1", "hello2"), True)
        self.assertEqual(check.execute("test1", "hello"), False)
        self.assertEqual(check.execute("hello", "hello"), False)

    def test_patterns_in_file_check(self):
        check = PatternsInFileCheck("test", [r"test\d", r"hello\dtest"])

        self.assertEqual(check.execute("test", "test1"), True)
        self.assertEqual(check.execute("test", "hello2test"), True)
        self.assertEqual(check.execute("hello", "test1"), False)
        self.assertEqual(check.execute("test", "TEST1"), False)
        self.assertEqual(check.execute("test", "hello3hello"), False)

    def test_patterns_tuple_in_file_check(self):
        check = PatternsInFileCheck(
            "test", [(r"test\d", re.IGNORECASE), (r"hello\dtest", 0)]
        )

        self.assertEqual(check.execute("test", "test1"), True)
        self.assertEqual(check.execute("test", "TEST2"), True)
        self.assertEqual(check.execute("test", "hello2test"), True)
        self.assertEqual(check.execute("hello", "test1"), False)
        self.assertEqual(check.execute("test", "hello3hello"), False)

    def test_pattern_in_files_check(self):
        check = PatternInFilesCheck(["test", "hello"], r"test\d")

        self.assertEqual(check.execute("1test1", "test1"), True)
        self.assertEqual(check.execute("1hello2", "test2"), True)
        self.assertEqual(check.execute("1foo2", "test2"), False)
        self.assertEqual(check.execute("1hello2", "hello2"), False)

    def test_pattern_in_file_pattern_check(self):
        check = PatternInFilePatternCheck(r"test|hello", r"foo|bar")

        self.assertEqual(check.execute("test1", "foo1"), True)
        self.assertEqual(check.execute("hello1", "bar1"), True)
        self.assertEqual(check.execute("foo1", "test1"), False)

    def test_pattern_in_file_pattern_flags_check(self):
        check = PatternInFilePatternCheck(
            r"test|hello", r"foo|bar", re.IGNORECASE, re.IGNORECASE
        )

        self.assertEqual(check.execute("test1", "foo1"), True)
        self.assertEqual(check.execute("test1", "FOO2"), True)
        self.assertEqual(check.execute("HELLO2", "bar1"), True)
        self.assertEqual(check.execute("foo1", "test1"), False)

    def test_patterns_in_file_pattern_check(self):
        check = PatternsInFilePatternCheck(r"test|hello", [r"foo\d", r"bar\d"])

        self.assertEqual(check.execute("test1", "foo1"), True)
        self.assertEqual(check.execute("hello1", "foo1"), True)

        self.assertEqual(check.execute("test1", "bar1"), True)
        self.assertEqual(check.execute("test1", "bar2"), True)

        self.assertEqual(check.execute("baz1", "bar2"), False)
        self.assertEqual(check.execute("test1", "baz1"), False)
        self.assertEqual(check.execute("baz1", "baz2"), False)

    def test_patterns_in_file_pattern_flags_check(self):
        check = PatternsInFilePatternCheck(
            r"test|hello",
            [(r"foo\d", 0), (r"bar\d", re.IGNORECASE)],
            re.IGNORECASE,
        )

        self.assertEqual(check.execute("test1", "foo1"), True)
        self.assertEqual(check.execute("HELLO", "foo1"), True)

        self.assertEqual(check.execute("test1", "bar1"), True)
        self.assertEqual(check.execute("test1", "BAR2"), True)

        self.assertEqual(check.execute("baz1", "bar2"), False)
        self.assertEqual(check.execute("test1", "BAZ1"), False)
        self.assertEqual(check.execute("baz1", "baz2"), False)

    def test_linguistic_exception_handler(self):
        checks = [FileCheck("test"), TextCheck("foo")]

        self.assertEqual(
            handle_linguistic_checks("test1", "foo1", checks), True
        )
        self.assertEqual(
            handle_linguistic_checks("test1", "bar1", checks), True
        )
        self.assertEqual(
            handle_linguistic_checks("hello1", "foo1", checks), True
        )
        self.assertEqual(
            handle_linguistic_checks("hello1", "bar1", checks), False
        )



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Iterable, List

from troubadix.plugin import FilePlugin, FilesPlugin, Plugin

from .badwords import CheckBadwords
from .copyright_text import CheckCopyrightText
from .copyright_year import CheckCopyrightYear
from .creation_date import CheckCreationDate
from .cve_format import CheckCVEFormat
from .cvss_format import CheckCVSSFormat
from .dependencies import CheckDependencies
from .dependency_category_order import CheckDependencyCategoryOrder
from .deprecated_dependency import CheckDeprecatedDependency
from .deprecated_functions import CheckDeprecatedFunctions
from .description import CheckDescription
from .double_end_points import CheckDoubleEndPoints
from .duplicate_oid import CheckDuplicateOID
from .duplicated_script_tags import CheckDuplicatedScriptTags
from .encoding import CheckEncoding
from .forking_nasl_functions import CheckForkingNaslFunctions
from .get_kb_on_services import CheckGetKBOnServices
from .grammar import CheckGrammar
from .http_links_in_tags import CheckHttpLinksInTags
from .illegal_characters import CheckIllegalCharacters
from .log_messages import CheckLogMessages
from .malformed_dependencies import CheckMalformedDependencies
from .misplaced_compare_in_if import CheckMisplacedCompareInIf
from .missing_desc_exit import CheckMissingDescExit
from .missing_tag_solution import CheckMissingTagSolution
from .newlines import CheckNewlines
from .overlong_script_tags import CheckOverlongScriptTags
from .prod_svc_detect_in_vulnvt import CheckProdSvcDetectInVulnvt
from .qod import CheckQod
from .reporting_consistency import CheckReportingConsistency
from .risk_factor import CheckRiskFactor
from .script_add_preference_type import CheckScriptAddPreferenceType
from .script_calls_empty_values import CheckScriptCallsEmptyValues
from .script_calls_recommended import CheckScriptCallsRecommended
from .script_category import CheckScriptCategory
from .script_copyright import CheckScriptCopyright
from .script_family import CheckScriptFamily
from .script_tag_form import CheckScriptTagForm
from .script_tag_whitespaces import CheckScriptTagWhitespaces
from .script_tags_mandatory import CheckScriptTagsMandatory
from .script_version_and_last_modification_tags import (
    CheckScriptVersionAndLastModificationTags,
)
from .script_xref_form import CheckScriptXrefForm
from .script_xref_url import CheckScriptXrefUrl
from .security_messages import CheckSecurityMessages
from .set_get_kb_calls import CheckWrongSetGetKBCalls
from .solution_text import CheckSolutionText
from .solution_type import CheckSolutionType
from .spelling import CheckSpelling
from .tabs import CheckTabs
from .todo_tbd import CheckTodoTbd
from .trailing_spaces_tabs import CheckTrailingSpacesTabs
from .using_display import CheckUsingDisplay
from .valid_oid import CheckValidOID
from .valid_script_tag_names import CheckValidScriptTagNames
from .variable_assigned_in_if import CheckVariableAssignedInIf
from .vt_file_permissions import CheckVTFilePermissions
from .vt_placement import CheckVTPlacement

# plugins checking single files
_FILE_PLUGINS = [
    CheckBadwords,
    CheckCopyrightText,
    CheckCopyrightYear,
    CheckCreationDate,
    CheckCVEFormat,
    CheckCVSSFormat,
    CheckDependencies,
    CheckDependencyCategoryOrder,
    CheckDeprecatedDependency,
    CheckDeprecatedFunctions,
    CheckDescription,
    CheckDoubleEndPoints,
    CheckDuplicatedScriptTags,
    CheckEncoding,
    CheckForkingNaslFunctions,
    CheckGetKBOnServices,
    CheckGrammar,
    CheckHttpLinksInTags,
    CheckIllegalCharacters,
    CheckLogMessages,
    CheckMalformedDependencies,
    CheckMisplacedCompareInIf,
    CheckMissingDescExit,
    CheckMissingTagSolution,
    CheckNewlines,
    CheckOverlongScriptTags,
    CheckProdSvcDetectInVulnvt,
    CheckQod,
    CheckReportingConsistency,
    CheckRiskFactor,
    CheckScriptAddPreferenceType,
    CheckScriptCallsEmptyValues,
    CheckScriptCallsRecommended,
    CheckScriptCategory,
    CheckScriptCopyright,
    CheckScriptFamily,
    CheckScriptTagForm,
    CheckScriptTagsMandatory,
    CheckScriptTagWhitespaces,
    CheckScriptVersionAndLastModificationTags,
    CheckScriptXrefForm,
    CheckScriptXrefUrl,
    CheckSecurityMessages,
    CheckSolutionText,
    CheckSolutionType,
    CheckTabs,
    CheckTodoTbd,
    CheckTrailingSpacesTabs,
    CheckUsingDisplay,
    CheckValidOID,
    CheckValidScriptTagNames,
    CheckVariableAssignedInIf,
    CheckVTFilePermissions,
    CheckVTPlacement,
    CheckWrongSetGetKBCalls,
]

# plugins checking all files
_FILES_PLUGINS = [
    CheckDuplicateOID,
    CheckSpelling,
]


class Plugins:
    def __init__(
        self,
        file_plugins: Iterable[FilePlugin] = None,
        files_plugins: Iterable[FilesPlugin] = None,
    ):
        self.file_plugins = tuple(file_plugins) or tuple()
        self.files_plugins = tuple(files_plugins) or tuple()

    def __len__(self) -> int:
        return len(self.files_plugins + self.file_plugins)

    def __iter__(self) -> Iterable[Plugin]:
        return iter(self.files_plugins + self.file_plugins)


class StandardPlugins(Plugins):
    def __init__(
        self,
        excluded_plugins: List[str] = None,
        included_plugins: List[str] = None,
    ) -> None:
        file_plugins = _FILE_PLUGINS
        files_plugins = _FILES_PLUGINS
        if excluded_plugins:
            file_plugins = self._exclude_plugins(excluded_plugins, file_plugins)
            files_plugins = self._exclude_plugins(
                excluded_plugins, files_plugins
            )

        if included_plugins:
            file_plugins = self._include_plugins(included_plugins, file_plugins)
            files_plugins = self._include_plugins(
                included_plugins, files_plugins
            )

        super().__init__(file_plugins=file_plugins, files_plugins=files_plugins)

    @staticmethod
    def _exclude_plugins(
        excluded: Iterable[str], plugins: Iterable[Plugin]
    ) -> List[Plugin]:
        return [
            plugin
            for plugin in plugins
            if plugin.__name__ not in excluded and plugin.name not in excluded
        ]

    @staticmethod
    def _include_plugins(
        included: Iterable[str], plugins: Iterable[Plugin]
    ) -> List[Plugin]:
        return [
            plugin
            for plugin in plugins
            if plugin.__name__ in included or plugin.name in included
        ]



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckWrongSetGetKBCalls(FilePlugin):
    name = "check_set_get_kb_calls"

    def run(self) -> Iterator[LinterResult]:
        """
        Checks a given file if it calls any of the following functions setting
        or getting KB entries in a wrong way like e.g. with too much or too
        less function parameters.

        - set_kb_item(name:"kb/key", value:"value");
        - replace_kb_item(name:"kb/key", value:"value");
        - get_kb_item("kb/key");
        - get_kb_list("kb/key");

        Wrong examples which needs to be reported:

        - set_kb_item("kb/key", value:"value");
        - replace_kb_item(name:"kb/key", "value");
        - replace_kb_item(name:"kb/key");
        - replace_kb_item(name:"kb/key", name:"kb/key");
        - get_kb_item(name:"kb/key");

        Args:
            nasl_file: The VT/Include that is going to be checked
            file_content: The content of the nasl_file

        """
        file_content = self.context.file_content
        param_re = re.compile(r"(name|value) ?:")

        set_matches = re.finditer(
            r"(set|replace)_kb_item\s*\(([^)]+)\)\s*;",
            file_content,
            re.MULTILINE,
        )
        if set_matches is not None:
            for set_match in set_matches:
                if set_match is not None and set_match.group(2) is not None:
                    set_param_match = re.findall(param_re, set_match.group(2))
                    if not set_param_match or len(set_param_match) != 2:
                        yield LinterError(
                            "The VT/Include is missing a 'name:' and/or "
                            f"'value:' parameter: {set_match.group(0)}",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )

        get_matches = re.finditer(
            r"get_kb_(item|list)\s*\(([^)]+)\)\s*;", file_content, re.MULTILINE
        )
        if get_matches is not None:
            for get_match in get_matches:
                if get_match is not None and get_match.group(2) is not None:
                    get_param_match = re.findall(param_re, get_match.group(2))
                    if get_param_match and len(get_param_match) > 0:
                        yield LinterError(
                            "The VT/Include is using a non-existent 'name:' "
                            f"and/or 'value:' parameter: {get_match.group(0)}",
                            file=self.context.nasl_file,
                            plugin=self.name,
                        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.malformed_dependencies import CheckMalformedDependencies

from . import PluginTestCase

here = Path(__file__).parent


class CheckMalformedDependenciesTestCase(PluginTestCase):
    def test_ok(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example.nasl", "example2.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok_1(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example.nasl,example2.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        expected_result = (
            "The script dependency value is malformed and contains a "
            "comma in the dependency value: 'example.nasl,example2.nasl'"
        )

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(results[0].message, expected_result)

    def test_nok_2(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example.nasl,example2.nasl",'
            ' "example3.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        expected_result = (
            "The script dependency value is malformed and contains a "
            "comma in the dependency value: 'example.nasl,example2.nasl'"
        )

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(results[0].message, expected_result)

    def test_nok_3(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example.nasl example2.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        expected_result = (
            "The script dependency value is malformed and contains "
            "whitespace within the dependency value: "
            "'example.nasl example2.nasl'"
        )

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(results[0].message, expected_result)

    def test_nok_4(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example.nasl, example2.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        expected_result_1 = (
            "The script dependency value is malformed and contains "
            "a comma in the dependency value: 'example.nasl, example2.nasl'"
        )

        expected_result_2 = (
            "The script dependency value is malformed and contains "
            "whitespace within the dependency value: "
            "'example.nasl, example2.nasl'"
        )

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertIsInstance(results[1], LinterError)
        self.assertEqual(results[0].message, expected_result_1)
        self.assertEqual(results[1].message, expected_result_2)

    def test_nok_5(self):
        path = here / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_dependencies("example .nasl", "example2.nasl");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=here
        )
        plugin = CheckMalformedDependencies(fake_context)

        results = list(plugin.run())

        expected_result = (
            "The script dependency value is malformed and contains "
            "whitespace within the dependency value: 'example .nasl'"
        )

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(results[0].message, expected_result)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper import is_ignore_file
from troubadix.helper.patterns import ScriptTag, get_script_tag_pattern

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

# nb: Those are files which are correctly using a log_message() to do e.g. some
# additional reporting for the user. This is a valid case which doesn't need to
# be changed. The example and template files shouldn't be checked at all because
# the plugin would throw an unnecessary error for them if e.g. not running from
# within the nasl/common folder. But all three files are just examples and don't
# need to be checked at all.
IGNORE_FILES = [
    "mssql_version.nasl",
    "pre2008/domino_default_db.nasl",
    "pre2008/oracle_tnslsnr_security.nasl",
    "pre2008/smtp_AV_42zip_DoS.nasl",
    "pre2008/shoutcast_version.nasl",
    "2017/gb_default_http_credentials_report.nasl",
    "2017/gb_dcetest_report.nasl",
    "2021/gb_ntp_mode6_response_check.nasl",
    "GSHB/GSHB_WMI_get_Shares.nasl",
    "GSHB/GSHB_WMI_Loginscreen.nasl",
    "GSHB/GSHB_WMI_CD-FD-User-only-access.nasl",
    "gb_dicom_service_ae_title_brute_force.nasl",
    "Policy/policy_controls_fail.nasl",
    "PCIDSS/PCI-DSS.nasl",
    "2016/gb_ssl_tls_weak_hash_algo.nasl",
    "2018/gb_unquoted_path_vulnerabilities_win.nasl",
    "2009/remote-net-hub-3com.nasl",
    "2015/gb_vnc_brute_force.nasl",
    "2012/gb_secpod_ssl_ciphers_weak_report.nasl",
    "GSHB/GSHB_Kompendium.nasl",
    "/policy_control_template.nasl",
    "/template.nasl",
    "/examples/test_ipv6_packet_forgery.nasl",
]


class CheckReportingConsistency(FileContentPlugin):
    name = "check_reporting_consistency"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks the consistency between log_message,
        security_message reporting function and
        the cvss base value.
        """
        if nasl_file.suffix == ".inc":
            return

        if is_ignore_file(nasl_file, IGNORE_FILES):
            return

        security_message = re.compile(
            r"^\s*[^#]?\s*security_message\s*\(.+?\)\s*;\s",
            re.MULTILINE | re.DOTALL,
        ).search(file_content)
        log_message = re.compile(
            r"^\s*[^#]?\s*log_message\s*\(.+?\)\s*;\s",
            re.MULTILINE | re.DOTALL,
        ).search(file_content)

        cvss_base_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_base = cvss_base_pattern.search(file_content)

        if not cvss_base:
            yield LinterError(
                "VT/Include has no cvss_base tag",
                file=nasl_file,
                plugin=self.name,
            )
            return

        if log_message and cvss_base.group("value") != "0.0":
            yield LinterError(
                "Tag cvss_base is not 0.0 use report function security_message",
                file=nasl_file,
                plugin=self.name,
            )

        if security_message and cvss_base.group("value") == "0.0":
            yield LinterError(
                "Tag cvss_base is 0.0 use report function log_message",
                file=nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterWarning
from troubadix.plugins import CheckTodoTbd

from . import PluginTestCase


class CheckTodoTbdTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"summary", value:"TBD");\n'
            'script_tag(name:"solution", value:"TODO");\n'
            'script_tag(name:"impact", value:"@todo");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ignore(self):
        path = Path("http_func.inc")
        content = (
            "#TBD"
            'script_tag(name:"summary", value:"A value");#TBD\n'
            'script_tag(name:"solution", value:"A value");\n'
            'script_tag(name:"impact", value:"A value");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_tbd(self):
        path = Path("some/file.nasl")
        content = (
            "##TBD\n"
            'script_tag(name:"summary", value:"A value");#TBD\n'
            'script_tag(name:"solution", value:"A value");\n'
            'script_tag(name:"impact", value:"A value");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)

        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[0].message,
        )
        self.assertEqual(
            results[0].line,
            1,
        )

        self.assertIsInstance(results[1], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[1].message,
        )
        self.assertEqual(
            results[1].line,
            2,
        )

    def test_todo(self):
        path = Path("some/file.nasl")
        content = (
            "##TODO\n"
            'script_tag(name:"summary", value:"A value");#TBD\n'
            'script_tag(name:"solution", value:"A value");\n'
            'script_tag(name:"impact", value:"A value");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)

        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[0].message,
        )
        self.assertEqual(results[0].line, 1)

        self.assertIsInstance(results[1], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[1].message,
        )
        self.assertEqual(results[1].line, 2)

    def test_at_todo(self):
        path = Path("some/file.nasl")
        content = (
            "##@todo\n"
            'script_tag(name:"summary", value:"A value");#@todo\n'
            'script_tag(name:"solution", value:"A value");\n'
            'script_tag(name:"impact", value:"A value");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)

        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[0].message,
        )
        self.assertEqual(results[0].line, 1)

        self.assertIsInstance(results[1], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[1].message,
        )
        self.assertEqual(results[1].line, 2)

    def test_mixed(self):
        path = Path("some/file.nasl")
        content = (
            "##TBD\n"
            'script_tag(name:"summary", value:"A value");#TODO\n'
            'script_tag(name:"solution", value:"A value");\n'
            'script_tag(name:"impact", value:"A value");##@todo\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content.splitlines()
        )
        plugin = CheckTodoTbd(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 3)

        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[0].message,
        )
        self.assertEqual(results[0].line, 1)

        self.assertIsInstance(results[1], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[1].message,
        )
        self.assertEqual(results[1].line, 2)

        self.assertIsInstance(results[2], LinterWarning)
        self.assertEqual(
            "VT contains #TODO/TBD/@todo keywords.",
            results[2].message,
        )
        self.assertEqual(results[2].line, 4)



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
import unittest
from contextlib import contextmanager
from pathlib import Path
from subprocess import SubprocessError
from typing import Generator
from unittest.mock import MagicMock, patch

from troubadix.standalone_plugins.version_updated import (
    check_version_updated,
    git,
)
from troubadix.standalone_plugins.version_updated import main as plugin_main
from troubadix.standalone_plugins.version_updated import parse_args


@contextmanager
def tempgitdir() -> Generator[Path, None, None]:
    cwd = Path.cwd()
    tempdir = tempfile.TemporaryDirectory()
    temppath = Path(tempdir.name)
    os.chdir(str(temppath))
    git("init", "-b", "main")
    git("config", "--local", "user.email", "max.mustermann@example.com")
    git("config", "--local", "user.name", "Max Mustermann")
    yield temppath
    tempdir.cleanup()
    os.chdir(str(cwd))


def setupgit(tmpdir: Path) -> None:
    test_file = tmpdir / "test.nasl"
    test_file.write_text(
        'script_version("2021-03-02T12:11:43+0000");”\n'
        'script_tag(name:"last_modification", '
        'value:"2021-03-02 12:11:43 +0000 (Tue, 02 Mar 2021)");\n'
    )
    git("add", str(test_file))
    git("commit", "-m", "test")


def change_nothing(tmpdir: Path) -> None:
    test_file = tmpdir / "test.nasl"
    test_file.write_text(
        'script_version("2021-03-02T12:11:43+0000");”\n'
        'script_tag(name:"last_modification", '
        'value:"2021-03-02 12:11:43 +0000 (Tue, 02 Mar 2021)");123\n'
    )
    git("add", str(test_file))
    git("commit", "-m", "test_nothing")


def change_version(tmpdir: Path):
    test_file = tmpdir / "test.nasl"
    test_file.write_text(
        'script_version("2021-03-02T12:11:43+0001");”\n'
        'script_tag(name:"last_modification", '
        'value:"2021-03-02 12:11:43 +0000 (Tue, 02 Mar 2021)");\n'
    )
    git("add", str(test_file))
    git("commit", "-m", "test_version")


def change_last_modification(tmpdir: Path):
    test_file = tmpdir / "test.nasl"
    test_file.write_text(
        'script_version("2021-03-02T12:11:43+0000");”\n'
        'script_tag(name:"last_modification", '
        'value:"2021-03-02 12:11:43 +0010 (Tue, 02 Mar 2021)");\n'
    )
    git("add", str(test_file))
    git("commit", "-m", "test_last_modification")


def change_version_and_last_modification(tmpdir: Path):
    test_file = tmpdir / "test.nasl"
    test_file.write_text(
        'script_version("2021-03-02T12:11:43+0001");\n'
        'script_tag(name:"last_modification", '
        'value:"2021-03-02 12:11:43 +0001 (Tue, 02 Mar 2021)");\n'
    )
    git("add", str(test_file))
    git("commit", "-m", "test_both")


def get_mocked_arguments():
    mock = MagicMock()
    mock.commit_range = "HEAD~1"
    mock.files = []

    return mock


class TestVersionChanged(unittest.TestCase):
    def test_change_nothing(self):
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_nothing(tmpdir)
            parsed_args = parse_args(["-c", "HEAD~1"])
            self.assertFalse(
                check_version_updated(
                    parsed_args.files, parsed_args.commit_range
                )
            )

    def test_change_version(self):
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_version(tmpdir)
            parsed_args = parse_args(["-c", "HEAD~1"])
            self.assertFalse(
                check_version_updated(
                    parsed_args.files, parsed_args.commit_range
                )
            )

    def test_change_last_modification(self):
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_last_modification(tmpdir)
            parsed_args = parse_args(["-c", "HEAD~1"])
            self.assertFalse(
                check_version_updated(
                    parsed_args.files, parsed_args.commit_range
                )
            )

    def test_change_both(self):
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_version_and_last_modification(tmpdir)
            parsed_args = parse_args(["-c", "HEAD~1"])
            self.assertTrue(
                check_version_updated(
                    parsed_args.files, parsed_args.commit_range
                )
            )

    def test_git_fail(self):
        with self.assertRaises(SubprocessError):
            git("bla")

    def test_git_ok(self):
        git("--version")

    def test_args_ok(self):
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            self.assertEqual(parse_args(["-c", "HEAD"]).commit_range, "HEAD")
            self.assertEqual(
                parse_args(["-c", "HEAD", "-f", "test.nasl"]).files,
                [Path("test.nasl")],
            )

    @patch("troubadix.standalone_plugins.version_updated.parse_args")
    def test_main_ok(self, mock_args):
        mock_args.return_value = get_mocked_arguments()
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_version_and_last_modification(tmpdir)

            exit_code = plugin_main()

            self.assertEqual(exit_code, 0)

    @patch("troubadix.standalone_plugins.version_updated.parse_args")
    def test_main_nok(self, mock_args):
        mock_args.return_value = get_mocked_arguments()
        with tempgitdir() as tmpdir:
            setupgit(tmpdir)
            change_version(tmpdir)

            exit_code = plugin_main()

            self.assertEqual(exit_code, 2)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterable, Iterator

import chardet

from troubadix.plugin import LineContentPlugin, LinterError, LinterResult

# Only the ASCII and extended ASCII for now... # https://www.ascii-code.com/
# CHAR_SET = r"[^\x00-\xFF]"
# Temporary only check for chars in between 7f-9f, like in the old Feed-QA...
CHAR_SET = r"[\x7F-\x9F]"


class CheckEncoding(LineContentPlugin):
    name = "check_encoding"

    def check_lines(
        self,
        nasl_file: Path,
        lines: Iterable[str],
    ) -> Iterator[LinterResult]:

        content = nasl_file.read_bytes()

        detection = chardet.detect(content)
        encoding = detection.get("encoding")
        if encoding and encoding not in [
            "ascii",
            "latin1",
            "ISO-8859-1",
        ]:
            yield LinterError(
                f"VT uses a wrong encoding. Detected "
                f"encoding is {encoding}.",
                file=nasl_file,
                plugin=self.name,
            )

        for index, line in enumerate(lines, 1):
            encoding = re.search(CHAR_SET, line)
            if encoding is not None:
                yield LinterError(
                    "Found invalid character",
                    file=nasl_file,
                    plugin=self.name,
                    line=index,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import LinterError
from troubadix.plugins.deprecated_dependency import CheckDeprecatedDependency

from . import PluginTestCase, TemporaryDirectory


class CheckDeprecatedDependencyTestCase(PluginTestCase):
    def setUp(self) -> None:
        self.tempdir = TemporaryDirectory()
        self.dir = Path(self.tempdir) / "foo"
        self.dir.mkdir(parents=True)
        self.dep = self.dir / "example.inc"
        self.dep.write_text(
            "script_category(ACT_ATTACK);\n exit(66);",
            encoding=CURRENT_ENCODING,
        )

        return super().setUp()

    def tearDown(self) -> None:
        self.tempdir.cleanup()

    def test_ok(self):
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            "script_category(ACT_ATTACK);"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = self.dir / "file.inc"
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_no_dependency(self):
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            "script_category(ACT_ATTACK);"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_deprecated(self):
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            "script_category(ACT_ATTACK);\n"
            "exit(66);"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_deprecated2(self):
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            "script_category(ACT_ATTACK);\n"
            'script_tag(name:"deprecated", value:TRUE);'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_dependency_missing(self):
        dependency = "example2.inc"
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar...");\n'
            'script_dependencies("example2.inc");\n'
            "script_category(ACT_SCANNER);"
        )

        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=self.dir
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            f"The script dependency {dependency} could "
            "not be found within the VTs.",
            results[0].message,
        )

    def test_deprecated_dependency(self):
        path = self.dir / "file.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar...");\n'
            'script_dependencies("example.inc");\n'
            "script_category(ACT_SCANNER);"
        )

        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content, root=self.dir
        )
        plugin = CheckDeprecatedDependency(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT depends on example.inc, which is marked as deprecated.",
            results[0].message,
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Iterator

from troubadix.helper import is_ignore_file
from troubadix.helper.patterns import get_script_tag_patterns

from ..plugin import FileContentPlugin, LinterError, LinterResult

# Arbitrary limit adopted from original step
VALUE_LIMIT = 3000

IGNORE_FILES = [
    "gb_nmap6_",
    "monstra_cms_mult_vuln",
    "gb_huawei-sa-",
    "lsc_options.nasl",
]


class CheckOverlongScriptTags(FileContentPlugin):
    """This steps checks if the script_tag summary, impact,
    affected, insight, vuldetect or solution of a given VT
    contains an overlong line within the value string.

    Background for this is that (e.g. auto generated LSCs)
    are created by parsing an advisory and the whole
    content is placed in such a tag which could be quite large.
    """

    name = "check_overlong_script_tags"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        if is_ignore_file(nasl_file, IGNORE_FILES):
            return

        script_tag_patterns = get_script_tag_patterns()
        for tag, pattern in script_tag_patterns.items():
            for match in pattern.finditer(file_content):
                if len(match.group("value")) > VALUE_LIMIT:
                    yield LinterError(
                        f"Tag {tag.value} is to long"
                        f" with {len(match.group('value'))} characters. "
                        f"Max {VALUE_LIMIT}",
                        file=nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
from abc import ABC, abstractmethod
from typing import Iterable, List, Tuple, Union


class LinguisticCheck(ABC):
    """Base class for all linguistic checks"""

    @abstractmethod
    def execute(self, file_path: str, correction: str):
        pass


class FileCheck(LinguisticCheck):
    """Checks wether the given file contains the specified file path"""

    def __init__(self, file: str) -> None:
        self.file = file

    def execute(self, file_path: str, correction: str) -> bool:
        return self.file in file_path


class FilesCheck(LinguisticCheck):
    """Checks wether the given file contains any of the specified file paths"""

    def __init__(self, files: List[str]) -> None:
        self.files = files

    def execute(self, file_path: str, correction: str):
        return any(_file in file_path for _file in self.files)


class FilePatternCheck(LinguisticCheck):
    """Checks wether the given file matches the specified pattern"""

    def __init__(self, file_pattern: str, flags: re.RegexFlag = 0) -> None:
        self.file_pattern = re.compile(file_pattern, flags=flags)

    def execute(self, file_path: str, correction: str):
        return bool(self.file_pattern.search(file_path))


class TextCheck(LinguisticCheck):
    """Checks wether the correction contains the specified text"""

    def __init__(self, text: str) -> None:
        self.text = text

    def execute(self, file_path: str, correction: str):
        return self.text in correction


class PatternCheck(LinguisticCheck):
    """Checks wether the correction matches the specified pattern"""

    def __init__(self, pattern: str, flags: re.RegexFlag = 0) -> None:
        self.pattern = re.compile(pattern, flags=flags)

    def execute(self, file_path: str, correction: str):
        return bool(self.pattern.search(correction))


class PatternsCheck(LinguisticCheck):
    """Checks wether the correction matches any of the specified patterns"""

    def __init__(
        self,
        patterns: Union[List[str], List[Tuple[str, re.RegexFlag]]],
    ) -> None:
        # Originally I tried isinstance(patterns, Tuple[...]) for testing,
        # but that throws an exception because Tuple[...]
        # is only valid for type hinting :/
        if isinstance(patterns[0], Tuple):
            self.patterns = [
                re.compile(pattern, flags=flags) for pattern, flags in patterns
            ]
        else:
            self.patterns = [re.compile(pattern) for pattern in patterns]

    def execute(self, file_path: str, correction: str):
        return any(
            bool(pattern.search(correction)) for pattern in self.patterns
        )


class CompositeCheck(LinguisticCheck):
    """
    Base class for checks that are composites of a number of other checks,
    which all have to pass for this check to pass
    """

    def __init__(self, *checks: Iterable[LinguisticCheck]) -> None:
        self.checks = checks

    def execute(self, file_path: str, correction: str):
        return all(
            check.execute(file_path, correction) for check in self.checks
        )


class TextInFileCheck(CompositeCheck):
    """Checks wether the correction contains the specified text
    and the given file contains the specified file path
    """

    def __init__(self, file_path: str, text: str) -> None:
        super().__init__(FileCheck(file_path), TextCheck(text))


class PatternInFileCheck(CompositeCheck):
    """Checks wether the correction matches the specified pattern
    and the file contains the specified file path
    """

    def __init__(
        self, file: str, pattern: str, flags: re.RegexFlag = 0
    ) -> None:
        super().__init__(FileCheck(file), PatternCheck(pattern, flags))


class PatternsInFileCheck(CompositeCheck):
    """Checks wether the correction matches any of the specified patterns
    and the file contains the specified file_path
    """

    def __init__(
        self,
        file: str,
        patterns: Union[List[str], List[Tuple[str, re.RegexFlag]]],
    ) -> None:
        super().__init__(FileCheck(file), PatternsCheck(patterns))


class PatternInFilesCheck(CompositeCheck):
    """Checks wether the correction matches the specified pattern
    and the file matches any of the specified file paths
    """

    def __init__(
        self, files: List[str], pattern: str, flags: re.RegexFlag = 0
    ) -> None:
        super().__init__(FilesCheck(files), PatternCheck(pattern, flags))


class PatternInFilePatternCheck(CompositeCheck):
    """Checks wether the correction matches the specified pattern
    and the file matches the specified file pattern
    """

    def __init__(
        self,
        file_pattern: str,
        text_pattern: str,
        file_pattern_flags: re.RegexFlag = 0,
        text_pattern_flags: re.RegexFlag = 0,
    ) -> None:
        super().__init__(
            FilePatternCheck(file_pattern, file_pattern_flags),
            PatternCheck(text_pattern, text_pattern_flags),
        )


class PatternsInFilePatternCheck(CompositeCheck):
    """Checks wether the correction matches any of the specified patterns
    and the file matches the specified file pattern
    """

    def __init__(
        self,
        file_pattern: str,
        patterns: Union[List[str], List[Tuple[str, re.RegexFlag]]],
        file_pattern_flags: re.RegexFlag = 0,
    ) -> None:
        super().__init__(
            FilePatternCheck(file_pattern, file_pattern_flags),
            PatternsCheck(patterns),
        )


def handle_linguistic_checks(
    file: str, correction: str, checks: Iterable[LinguisticCheck]
) -> bool:
    """Determinates if any of the provided checks pass
    for the provided file and correction

    Args:
        file (str): The file to pass to all checks
        correction (str): The correction to pass to all checks
        checks (Iterable[LinguisticCheck]): The checks that have to be passed

    Returns:
        bool: Wether any check was passed by the provided file and correction
    """
    return any(check.execute(file, correction) for check in checks)



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from typing import Iterator

from troubadix.helper.patterns import ScriptTag, _get_tag_pattern
from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckSolutionText(FilePlugin):
    name = "check_solution_text"

    def run(self) -> Iterator[LinterResult]:
        """There are specific guidelines on the syntax for the solution tag on
        VTs with the solution_type "NoneAvailable" or "WillNotFix" available at:

        https://community.greenbone.net/t/vt-development/226 (How to handle VTs
        with "no solution" for the user)

        This script checks if those guidelines are upheld.

        Args:
            nasl_file: Name of the VT to be checked
            file_content: Content of the nasl_file to be checked

        """
        # Two different strings, one for RegEx one for output
        correct_none_available_pattern = (
            r"script_tag\s*\("
            r'\s*name\s*:\s*"solution"\s*,'
            r'\s*value\s*:\s*"No\s+known\s+solution\s+is\s+available\s+as\s+of'
            r"\s+(0[1-9]|[12][0-9]|3[01])(st|nd|rd|th)\s+(January|February|"
            r"March|April|May|June|July|August|September|October|November|"
            r"December),\s+20[0-9]{2}\.\s+Information\s+regarding\s+this\s+"
            r"issue\s+will\s+be\s+updated\s+once\s+solution\s+details\s+"
            r"are\s+available\."
        )
        correct_none_available_syntax = (
            '  script_tag(name:"solution", '
            'value:"No known solution is '
            "available as of dd(st|nd|rd|th) mmmmmmmm, yyyy.\n  "
            "Information regarding this issue will be updated once solution "
            'details are available.");'
        )

        # same here
        correct_will_not_fix_pattern = (
            r"script_tag\s*\("
            r'\s*name\s*:\s*"solution"\s*,\s*value\s*:\s*"(No\s+solution\s+'
            r"(was\s+made\s+available\s+by\s+the\s+vendor|is\s+required)\."
            r"\s+(Note|Vendor statement):.+|(No\s+solution\s+was\s+made\s+"
            r"available\s+by\s+the\s+vendor|No\s+known\s+solution\s+was\s+made"
            r"\s+available\s+for\s+at\s+least\s+one\s+year\s+since\s+the\s+"
            r"disclosure\s+of\s+this\s+vulnerability\.\s+Likely\s+none\s+will"
            r"\s+be\s+provided\s+anymore)\.\s+General\s+solution\s+options\s+"
            r"are\s+to\s+upgrade\s+to\s+a\s+newer\s+release,\s+disable\s+"
            r"respective\s+features,\s+remove\s+the\s+product\s+or\s+replace\s+"
            r"the\s+product\s+by\s+another\s+one\.)"
        )
        correct_will_not_fix_syntax = (
            '  script_tag(name:"solution", '
            'value:"No known solution was made '
            "available for at least one year\n  since the disclosure of this "
            "vulnerability. Likely none will be provided anymore. General "
            "solution\n  options are to upgrade to a newer release, disable "
            "respective features, remove the product or\n  replace the "
            'product by another one.");\n\n'
            '  script_tag(name:"solution", '
            'value:"No solution was made '
            "available by the vendor. General solution\n  options are to "
            "upgrade to a newer release, disable respective features, remove "
            'the product or\n  replace the product by another one.");\n\n'
            '  script_tag(name:"solution", '
            'value:"No solution was made '
            "available by the vendor.\n\n  Note: "
            '<add a specific note for the reason here>.");\n\n'
            '  script_tag(name:"solution", '
            'value:"No solution was made '
            "available by the vendor.\n\n  Vendor statement: "
            '<add specific vendor statement here>.");\n\n'
            '  script_tag(name:"solution", '
            'value:"No solution is required.\n\n  Note: <add a specific note '
            'for the reason here, e.g. CVE was disputed>.");'
        )
        file_content = self.context.file_content

        if _get_tag_pattern(
            name=ScriptTag.SOLUTION_TYPE.value, value="NoneAvailable"
        ).search(file_content) and not re.search(
            correct_none_available_pattern, file_content
        ):
            yield LinterError(
                "The VT with solution type 'NoneAvailable' is using an "
                "incorrect syntax in the solution text. Please use "
                f"(EXACTLY):\n{correct_none_available_syntax}",
                file=self.context.nasl_file,
                plugin=self.name,
            )
        elif _get_tag_pattern(
            name=ScriptTag.SOLUTION_TYPE.value, value="WillNotFix"
        ).search(file_content) and not re.search(
            correct_will_not_fix_pattern, file_content
        ):
            yield LinterError(
                "The VT with solution type 'WillNotFix' is using an incorrect "
                "syntax in the solution text. Please use one of these "
                f"(EXACTLY):\n{correct_will_not_fix_syntax}",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from unittest.mock import MagicMock

from troubadix.plugin import LinterError
from troubadix.plugins.creation_date import CheckCreationDate

from . import PluginTestCase


class CheckCreationDateTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2013-05-14 11:24:55 '
            '+0200 (Tue, 14 May 2013)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_missing(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"7.5");\n'
            'script_tag(name:"cvss_base_vector", '
            'value:"AV:N/AC:L/Au:N/C:P/I:P/A:P");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "No creation date has been found.",
            results[0].message,
        )

    def test_wrong_weekday(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2013-05-14 11:24:55 +0200 '
            '(Mon, 14 May 2013)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Wrong day of week. Please change it from 'Mon' to 'Tue'.",
            results[0].message,
        )

    def test_no_timezone(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2013-05-14 11:24:55 '
            '(Tue, 14 May 2013)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "False or incorrectly formatted creation_date.",
            results[0].message,
        )

    def test_different_dates(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2013-05-14 11:24:55 +0200 '
            '(Tue, 15 May 2013)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The creation_date consists of two different dates.",
            results[0].message,
        )

    def test_wrong_length(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2013-05-14 11:24:55 +0200 '
            '(Tue, 14 May 2013 )");'
        )
        fake_context = MagicMock()
        fake_context.nasl_file = path
        fake_context.file_content = content
        plugin = CheckCreationDate(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "False or incorrectly formatted creation_date.",
            results[0].message,
        )



# Copyright (c) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from unittest.mock import MagicMock

from troubadix.plugin import LinterError
from troubadix.plugins.duplicate_oid import CheckDuplicateOID

from . import PluginTestCase

here = Path(__file__).parent


class CheckDuplicateOIDTestCase(PluginTestCase):
    def test_ok(self):
        file1 = here / "test_files" / "nasl" / "21.04" / "fail.nasl"
        file2 = (
            here / "test_files" / "nasl" / "21.04" / "fail_name_newline.nasl"
        )
        context = MagicMock()
        context.nasl_files = [file1, file2]
        context.root = here
        plugin = CheckDuplicateOID(context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok_no_script_oid(self):
        file1 = (
            here / "test_files" / "nasl" / "21.04" / "fail_name_newline.nasl"
        )
        file2 = here / "test_files" / "nasl" / "21.04" / "fail_badwords.nasl"
        context = MagicMock()
        context.nasl_files = [file1, file2]
        context.root = here
        plugin = CheckDuplicateOID(context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Could not find an OID"
            " in 'test_files/nasl/21.04/fail_badwords.nasl'.",
            results[0].message,
        )

    def test_duplicate_oid_function(self):
        file1 = here / "test_files" / "nasl" / "21.04" / "fail.nasl"
        file2 = here / "test_files" / "nasl" / "21.04" / "test.nasl"
        context = MagicMock()
        context.nasl_files = [file1, file2]
        context.root = here
        plugin = CheckDuplicateOID(context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "OID 1.3.6.1.4.1.25623.1.0.100312 is duplicated in the following"
            " files:\n"
            "test_files/nasl/21.04/test.nasl\n"
            "test_files/nasl/21.04/fail.nasl",
            results[0].message,
        )

    def test_invalid_oid(self):
        file2 = (
            here
            / "test_files"
            / "nasl"
            / "21.04"
            / "fail_name_and_copyright_newline.nasl"
        )
        context = MagicMock()
        context.nasl_files = [file2]
        context.root = here
        plugin = CheckDuplicateOID(context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Invalid OID 1.2.3.4.5.6.78909.8.7.654321 found"
            " in 'test_files/nasl/21.04/fail_name_and_copyright_newline.nasl'.",
            results[0].message,
        )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import (
    SpecialScriptTag,
    get_special_script_tag_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

VALID_FAMILIES = [
    "AIX Local Security Checks",
    "AlmaLinux Local Security Checks",
    "Amazon Linux Local Security Checks",
    "Brute force attacks",
    "Buffer overflow",
    "CISCO",
    "CentOS Local Security Checks",
    "Citrix Xenserver Local Security Checks",
    "Compliance",
    "Credentials",
    "Databases",
    "Debian Local Security Checks",
    "Default Accounts",
    "Denial of Service",
    "F5 Local Security Checks",
    "FTP",
    "Fedora Local Security Checks",
    "FortiOS Local Security Checks",
    "FreeBSD Local Security Checks",
    "Gain a shell remotely",
    "General",
    "Gentoo Local Security Checks",
    "Huawei",
    "Huawei EulerOS Local Security Checks",
    "HP-UX Local Security Checks",
    "IT-Grundschutz",
    "IT-Grundschutz-deprecated",
    "IT-Grundschutz-15",
    "JunOS Local Security Checks",
    "Mac OS X Local Security Checks",
    "Mageia Linux Local Security Checks",
    "Malware",
    "Mandrake Local Security Checks",
    "Nmap NSE",
    "Nmap NSE net",
    "Oracle Linux Local Security Checks",
    "PCI-DSS",
    "PCI-DSS 2.0",
    "Palo Alto PAN-OS Local Security Checks",
    "Peer-To-Peer File Sharing",
    "Policy",
    "Port scanners",
    "Privilege escalation",
    "Product detection",
    "RPC",
    "Red Hat Local Security Checks",
    "Remote file access",
    "Rocky Linux Local Security Checks",
    "SMTP problems",
    "SNMP",
    "SSL and TLS",
    "Service detection",
    "Settings",
    "Slackware Local Security Checks",
    "Solaris Local Security Checks",
    "SuSE Local Security Checks",
    "Ubuntu Local Security Checks",
    "Useless services",
    "VMware Local Security Checks",
    "Web Servers",
    "Web application abuses",
    "Windows",
    "Windows : Microsoft Bulletins",
]


class CheckScriptFamily(FileContentPlugin):
    name = "check_script_family"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks VT for the existence / validity
        of its script family"""
        if nasl_file.suffix == ".inc":
            return

        family_pattern = get_special_script_tag_pattern(SpecialScriptTag.FAMILY)
        matches = list(family_pattern.finditer(file_content))

        if not matches:
            yield LinterError(
                "No script family exist",
                file=nasl_file,
                plugin=self.name,
            )
            return

        if len(matches) > 1:
            yield LinterError(
                "More then one script family exist",
                file=nasl_file,
                plugin=self.name,
            )
            return

        if matches[0].group("value") not in VALID_FAMILIES:
            yield LinterError(
                "Invalid or misspelled script family "
                f"'{matches[0].group('value')}'",
                file=nasl_file,
                plugin=self.name,
            )



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.http_links_in_tags import CheckHttpLinksInTags

from . import PluginTestCase


class CheckHttpLinksInTagsTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'get_app_port_from_cpe_prefix("cpe:/o:foo:bar");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckHttpLinksInTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckHttpLinksInTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_not_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar. '
            'https://www.website.de/demo");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckHttpLinksInTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "One script_tag in the VT is using a "
            "HTTP link/URL which should be moved to a separate "
            '\'script_xref(name:"URL", value:"");\' tag instead: '
            '\'script_tag(name:"summary", value:"Foo Bar. '
            "https://www.website.de/demo\");'",
            results[0].message,
        )

    def test_not_ok2(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'script_xref(name:"URL", '
            'value:"https://nvd.nist.gov/vuln/detail/CVE-1234");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckHttpLinksInTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The following script_xref is pointing "
            "to Mitre/NVD which is already covered by the script_cve_id. "
            "This is a redundant info and the script_xref needs to be "
            'removed: script_xref(name:"URL", '
            'value:"https://nvd.nist.gov/vuln/detail/CVE-1234");',
            results[0].message,
        )

    def test_http_link_in_tags_ok(self):
        testcases = [
            "01. The payloads try to open a connection to www.google.com",
            "02. The script attempts to connect to www.google.com",
            "03. to retrieve a web page from www.google.com",
            "04. Subject: commonName=www.paypal.com",
            "05. Terms of use at https://www.verisign.com/rpa",
            "06. example.com",
            "07. example.org",
            "08. www.exam",
            "09. sampling the resolution of a name (www.google.com)",
            "10. once with 'www.' and once without",
            "11. wget http://www.javaop.com/~ron/tmp/nc",
            "12. Ncat: Version 5.30BETA1 (http://nmap.org/ncat)",
            "13. as www.windowsupdate.com. (BZ#506016)",
            "14. located at http://sambarserver/session/pagecount.",
            "15. http://rest.modx.com",
            "16. ftp:// ",
            "17. ftp://'",
            "18. ftp://)",
            "19. ftp.c",
            "20. ftp.exe",
            "21. using special ftp://",
            "22. running ftp.",
            "23. ftp. The vulnerability",
            "24. 'http://' protocol",
            "25. handle <a href='http://...'> properly",
            "26. Switch to git+https://",
            "27. wget https://compromised-domain.com/important-file",
            "28. the https:// scheme",
            "29. https://www.phishingtarget.com@evil.com",
            "30. 'http://'",
            "31. 'https://'",
            "32. distributions on ftp.proftpd.org have all been",
            "33. information from www.mutt.org:",
            "34. According to www.tcpdump.org:",
            "35. According to www.kde.org:",
            "36. From the www.info-zip.org site:",
            # pylint: disable=line-too-long
            "37.  (www.isg.rhul.ac.uk) for discovering this flaw and Adam Langley and",
            "38. Sorry about having to reissue this one -- I pulled it from ftp.gnu.org not",
            "39. http://internal-host$1 is still insecure",
            "40. from online sources (ftp://, http:// etc.).",
            "41. this and https:// and that.",
            "42. such as 'http://:80'",
        ]

        for testcase in testcases:
            self.assertTrue(CheckHttpLinksInTags.check_to_continue(testcase))

    def test_http_link_in_tags_not_ok(self):
        testcases = [
            "The payloads try to open a connection to www.bing.com",
            "examplephishing.org",
            "located at http://sambdadancinglessions/session/pagecount.",
            "fdp:// ",
            "Switch to svn+https://",
            "greenbone.net",
        ]

        for testcase in testcases:
            self.assertFalse(CheckHttpLinksInTags.check_to_continue(testcase))
